  // [this function was updated 10/20/98 to make it portable]
  //
  // hotkeys( on_off ) makes the terminal keyboard "hot"
  // by changing certain elements in the termios structure
  // [see man termios], thus allowing single-character key-presses to
  // be immediately read while a program is running.  in particular,
  // hotkeys(on) does the following:
  //
  // (a) turns off the ISIG bit of c_lflag, so that interrupts like
  // ^C (INTR) or ^Z (SUSP) or ^\ (QUIT) are disabled.
  //
  // (b) turns off the ICANON bit of c_lflag, which puts the
  // terminal in the "non-canonical" mode, so that "read" requests
  // [using read()] are satisfied immediately when a key is
  // pressed, without having to wait for a newline character.  (in
  // the "canonical" mode, input is read only after a newline
  // character is received).
  //
  // (c) turns off the ECHO bit of c_lflag, so that characters
  // entered at the keyboard are not echoed on the terminal screen.
  //
  // (d) turns off the IXON bit of c_iflag, so that ^S (STOP) and
  // ^Q (START) are disabled.
  //
  // (e) sets c_cc[VMIN] = 0, so that a "read" request will be
  // satisfied even if there are no characters to read.  (most of
  // the time there will be no chars to read).  (ordinarily,
  // c_cc[VMIN] = 1, although on some machines c_cc[VMIN] = 4, for
  // some reason i don't fathom.)
  //
  // (f) sets c_cc[VTIME] = 1, so if there is a "read" request, the
  // system will wait for 0.1  second for the character to be read.
  // in this situation (that is, when MIN is also 0), a "read" will
  // be satisfied as soon as a single character is recieved OR if
  // the (0.1-second)  timer expires, in which case a 0 is returned
  // by read.  for "fast" terminals like X terminals, c_cc[VTIME]
  // may be set to 0, but for "slow" terminals like graphons, it
  // helps to provide time for the "read" request to operate, and
  // 0.1  second is the shortest time available.  to prevent a
  // program from stuttering along with 0.1-second  blocks, it is
  // suggested that "read" requests be allowed to occur (by the
  // calling program) only at "longer" intervals.  what is meant by
  // "longer" is hard to say.  in our programs we allow this
  // "blocking time" to be adjusted by the user so as to give
  // optimum time sharing between graph display and looking for
  // characters from the keyboard.  it's kind of a crude
  // event-handling mechanism, but it works.  NOTE: it may help to
  // insert a fflush(stdout) command in any loop of the calling
  // program, in order to provide access to any "read" statement.

#include "gogo.h"


void hotkeys(off_on k)
{
    static bool hot = false;   // flag to tell whether hotkeys are on or off
    static tio t;              // tio: struct termios
    static tio sv;
    int bits = ICANON | ECHO | ISIG ;

    if (k){                             // turn hot keys on...
        if (hot){                       // but if they are currently on,
            return;                     // leave them on
        }
        else {                          // turn them on
            tcgetattr(0, &t);           // get default settings into t
            sv = t;                     // save default settings
            t.c_lflag &= ~bits;         // disable interupts, echo & icanon
            t.c_iflag &= ~IXON;         // disable ^S and ^Q
            t.c_cc[VMIN] = 0;           // 'read' reads a min of zero chars
            t.c_cc[VTIME] = 1;          // 'read' blocks for 0.1 second
            tcsetattr(0, TCSANOW, &t);  // set above settings
            hot = true;
        }
    }
    else {                              // turn hotkeys off
        if (hot){
            tcsetattr(0, TCSANOW, &sv); // reset default settings
            hot = false;
        }
        else {                          // keys already cold
            return;                     // so don't do anything
        }
    }
}
